@@ -1,5 +1,7 @@
 from apimatic_core_interfaces.types.authentication import Authentication
 
+from apimatic_core.utilities.auth_helper import AuthHelper
+
 
 class QueryAuth(Authentication):
 
@@ -7,8 +9,7 @@ class QueryAuth(Authentication):
         self._auth_params = auth_params
 
     def is_valid(self):
-        return self._auth_params and all(param and self._auth_params[param] for param in self._auth_params)
+        return AuthHelper.is_valid_auth(self._auth_params)
 
     def apply(self, http_request):
-        for param in self._auth_params:
-            http_request.add_query_parameter(param, self._auth_params[param])
+        return AuthHelper.apply(self._auth_params, http_request.add_query_parameter)
